var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "emitter-tiny"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var emitter_tiny_1 = __importDefault(require("emitter-tiny"));
    var TinyPromiseQueue = /** @class */ (function () {
        function TinyPromiseQueue(factoryArr, cap, errorPolicy) {
            if (factoryArr === void 0) { factoryArr = []; }
            if (cap === void 0) { cap = 4; }
            if (errorPolicy === void 0) { errorPolicy = 'abort'; }
            var _this = this;
            this.factoryArr = factoryArr;
            this.cap = cap;
            this.errorPolicy = errorPolicy;
            this.pending = [];
            this.isAbort = false;
            this.event = new emitter_tiny_1.default();
            this.total = 0;
            this.response = [];
            this.abort = function () {
                _this.isAbort = true;
                _this.pending.forEach(function (i) { var _a; return (_a = i.abort) === null || _a === void 0 ? void 0 : _a.call(i); });
                _this.pending = [];
                _this.event.emit('fail', new Error('you call the abort method.'));
            };
            this.next = function (promise) {
                var _a;
                if (_this.isAbort)
                    return;
                _this.pending = _this.pending.filter(function (item) { return item.promise !== promise; });
                (_a = _this.onProgress) === null || _a === void 0 ? void 0 : _a.call(_this, {
                    done: _this.total - _this.factoryArr.length - _this.pending.length,
                    total: _this.total
                });
                var nextFn = _this.factoryArr.shift();
                _this.execte(nextFn);
            };
            this.wrapper = function (promise) {
                promise
                    .then(function (res) {
                    _this.response.push(res);
                    setTimeout(function () {
                        _this.next(promise);
                    });
                    return res;
                })
                    .catch(function (err) {
                    if (_this.errorPolicy === 'abort') {
                        _this.abort();
                        setTimeout(function () { return _this.event.emit('fail', err); });
                        return err;
                    }
                    if (_this.errorPolicy === 'ignore') {
                        _this.response.push(err);
                        setTimeout(function () {
                            _this.next(promise);
                        });
                        return Promise.resolve(err);
                    }
                });
            };
            this.execte = function (fn) {
                var result = fn === null || fn === void 0 ? void 0 : fn();
                if (Array.isArray(result)) {
                    var promise = result[0], abort = result[1];
                    _this.wrapper(promise);
                    _this.pending.push({
                        promise: promise,
                        abort: abort
                    });
                }
                if (result instanceof Promise) {
                    _this.wrapper(result);
                    _this.pending.push({
                        promise: result
                    });
                }
                if (!_this.pending.length
                    && !_this.factoryArr.length) {
                    _this.event.emit('success', true);
                }
            };
            this.start = function () {
                var cap = _this.cap;
                _this.isAbort = false;
                _this.factoryArr = __spreadArray([], _this.factoryArr, true);
                var fnArr = _this.factoryArr.splice(0, cap);
                for (var _i = 0, fnArr_1 = fnArr; _i < fnArr_1.length; _i++) {
                    var fn = fnArr_1[_i];
                    _this.execte(fn);
                }
                return new Promise(function (res, rej) {
                    _this.event.on('success', function () { return res(_this.response); });
                    _this.event.on('fail', function (e) { return rej(e); });
                });
            };
            this.total = factoryArr.length;
        }
        return TinyPromiseQueue;
    }());
    exports.default = TinyPromiseQueue;
});
